/*
 *  This program is intended to be a quick benchmark of overlapped
 *   read and write operations in Wine.
 *
 *  It takes 90 samples of the overlapped read and write time, and
 *    reports the average time waiting for the operation to complete.
 *
 *  The time between a ReadFile/WriteFile call and the overlapped event
 *   flag is averaged after it becomes stable. (Initially, the wineserver
 *   does some caching of handles.)
 *
 *  The times are intended to be compared with the output of the same
 *   program, run on the same hardware, with changes to the file I/O code
 *   applied to wine.
 *
 *  This program assumes that there is a modem attached to COM1.
 *
 *  Mike McCormack  11 Mar 2001
 */

#include <stdio.h>
#include <sys/time.h>
#include <unistd.h>
#include "windows.h"

#define TOTAL_SAMPLES 100
#define DISCARD_SAMPLES 5

char attention[] ="AT\015";
char response[100];

int elapsed(struct timeval *start, struct timeval *end)
{
    return (end->tv_sec-start->tv_sec)*1000000 + end->tv_usec - start->tv_usec;
}

int main()
{
    HANDLE hComm;
    OVERLAPPED read_ov, write_ov;
    DWORD write_count, read_count;
    DWORD write_average, write_samples, read_average, read_samples;
    DWORD r,i;
    COMMTIMEOUTS timeouts;
    struct timeval write_start,write_end,read_start,read_end;

    hComm = CreateFile("COM1", GENERIC_READ|GENERIC_WRITE, 
                       0, NULL, 0, FILE_FLAG_OVERLAPPED, NULL);   
    if(!hComm)
        fprintf(stderr,"Unable to open COM1\n");

    /* set the comm timeouts */
    timeouts.ReadIntervalTimeout = 1000;
    timeouts.ReadTotalTimeoutMultiplier = 0;
    timeouts.ReadTotalTimeoutConstant = 200;
    timeouts.WriteTotalTimeoutMultiplier = 0;
    timeouts.WriteTotalTimeoutConstant = 200;
    SetCommTimeouts(hComm, &timeouts);

    ZeroMemory(&write_ov, sizeof write_ov);
    write_ov.hEvent = CreateEventA(NULL,FALSE,FALSE,NULL);
    write_count = 0;

    ZeroMemory(&read_ov, sizeof read_ov);
    read_ov.hEvent = CreateEventA(NULL,FALSE,FALSE,NULL);
    read_count=0;

    write_average=0;
    write_samples=0;
    read_average=0;
    read_samples=0;

    for(i=0; i<TOTAL_SAMPLES; i++)
    {
        /* write the modem query string */
        ResetEvent(write_ov.hEvent);
        gettimeofday(&write_start,NULL);
        r=WriteFile(hComm, attention, 3, &write_count, &write_ov);
        if(!r)
        {
            if (GetLastError()==ERROR_IO_PENDING)
            {
                r = WaitForSingleObject(write_ov.hEvent,INFINITE);
                gettimeofday(&write_end,NULL);
                if((r != WAIT_OBJECT_0)||(read_ov.InternalHigh==0))
                    fprintf(stderr,"W");
                else if(i>=DISCARD_SAMPLES)
                {
                    fprintf(stderr,"%d ", elapsed(&write_start,&write_end));
                    write_average += elapsed(&write_start,&write_end);
                    write_samples ++;
                }
            }
            else
                printf("write: error %lx\n",GetLastError());
        }
        else
            fprintf(stderr,"write: Overlapped write failed\n");
    
        ResetEvent(read_ov.hEvent);
        gettimeofday(&read_start,NULL);
        r=ReadFile(hComm, response, 7, &read_count, &read_ov);
        if(!r)
        {
            DWORD error = GetLastError();
            if (error==ERROR_IO_PENDING)
            {
                r = WaitForSingleObject(read_ov.hEvent,INFINITE);
                gettimeofday(&read_end,NULL);
                if((r != WAIT_OBJECT_0)||(read_ov.InternalHigh==0))
                    fprintf(stderr,"R");
                else if(i>=DISCARD_SAMPLES)
                {
                    fprintf(stderr,"%d ", elapsed(&read_start,&read_end));
                    read_average += elapsed(&read_start,&read_end);
                    read_samples ++;
                }
            }
            else
                printf("read: error %lx\n",error);
        }
        else
            printf("read: Overlapped failed\n");
     
        Sleep(100);
    }

    if(write_samples)
        printf("\n\nAverage write time was: %f (%ld samples)\n",
             (float)write_average/(float)write_samples,write_samples);
    
    if(read_samples)
        printf("\n\nAverage read time was: %f (%ld samples)\n",
             (float)read_average/(float)read_samples,read_samples);
    
    CloseHandle(hComm);
    return 0;
}

